package jeliot.gui;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import jeliot.parser.*;

/**
  * @author Pekka Uronen
  *
  * created 30.8.1999
  */
public class ProgramView extends ProgramTreeVisitor {

    private Hashtable nodes = new Hashtable();
    private DefaultMutableTreeNode progNode =
            new DefaultMutableTreeNode("Program");
    private JTree tree = new JTree(progNode); {
        tree.putClientProperty("JTree.lineStyle", "Angled");
    }

    public ProgramView(PCompilationUnit cu) {
        cu.acceptVisitor(this);
    }
    
    public JPanel createPanel() {

        Enumeration enum = progNode.preorderEnumeration();
        while (enum.hasMoreElements()) {
            DefaultMutableTreeNode node =
                    (DefaultMutableTreeNode)enum.nextElement();
            TreePath tp = new TreePath(node.getPath());
            tree.expandPath(tp);
        }
        
        JPanel p = new JPanel();
        p.setLayout(new BorderLayout());
        p.add("Center", new JScrollPane(tree));
        return p;
    }
            
    public void visitCompilationUnit(PCompilationUnit cu) {
        nodes.put(cu, progNode);
    }
        
    public void visitClass(final PClass p) {
        putNode(p, new DefaultMutableTreeNode(
            new PNode() {
                public String toString() {
                    return "class " + p.getName();
                }
            }
        ));
    
    }

    private void putNode(PElement p, DefaultMutableTreeNode node) {
        DefaultMutableTreeNode pano =
                (DefaultMutableTreeNode)nodes.get(p.getParent());
        if (pano != null) {
            pano.add(node);
        }
        else {
            progNode.add(node);
        }
        nodes.put(p, node);  
    }
    
    public void visitMethod(final PMethod p) {
        putNode(p, new DefaultMutableTreeNode(
            new PNode() {
                public String toString() {
                    return "method " + p.getName();
                }
            }
        ));
    }

    public void visitStatement(final PStatement p) {
        putNode(p, new DefaultMutableTreeNode(
            new PNode() {
                public String toString() {
                    return p.toString();
                }
            }
        ));
    }
    
    public void enterExpression(final PExpression p) {
        putNode(p, new DefaultMutableTreeNode(
            new PNode() {
                public String toString() {
                    return p.toString();
                }
            }
        ));
    }
    
    public void enterVariableDeclarator
            (final PVariableDeclarator p) {
                
        putNode(p, new DefaultMutableTreeNode(
            new PNode() {
                public String toString() {
                    return p.toString();
                }
            }
        ));
    }
    
}

class PNode {

}
