package jeliot.lang;

import java.lang.reflect.*;
import jeliot.parser.*;
import jeliot.theatre.*;

/**
  * @author Pekka Uronen
  *
  * created 29.8.1999
  */
public class DomesticMethodPointer extends MethodPointer {

    /** The method represented by this pointer.
      */
    private PMethod myMethod;
    
    public DomesticMethodPointer(PMethod meth) {
        this.myMethod = meth;
    }

    public Type getReturnType() {
        return myMethod.getType();
    }

    public boolean isStatic() {
        return myMethod.getModifiers().hasModifier(PModifiers.STATIC);
    }

    public boolean isPrivate() {
        return myMethod.getModifiers().hasModifier(PModifiers.PRIVATE);
    }

    /** default invocation code */
    public Return invoke(Instance ins, Value[] args,
            PMethodInvocation invo, Director dir) {

        if (isStatic()) {
            // static invocation
            return dir.animateSDMInvocation(this, args);
        }
        return null;
    }
    
    public ReferenceType getDeclaringClass() {
        return myMethod.getDeclaringClass().getType();
    }

    public Return doInvoke(Director director, Value[] args) {
        return myMethod.execute(director, args);
    }
    
    public String getName() {
        return myMethod.getName();
    }
    
        
}
