package jeliot.lang;

import java.util.*;
import jeliot.parser.*;
import jeliot.parser.error.*;
import jeliot.FeatureNotImplementedException;

/**
  * @author Pekka Uronen
  *
  * created         27.9.1999
  */
public class DomesticType extends ReferenceType {
       
    private PClass myClass;
    
    private DomesticType() { }
    
    public DomesticType(PClass c) {
        this.myClass = c;
    }
    
    /** Checks if a value of this type may be assigned to a variable of
      * given type t.
      */
    public boolean mayBeAssignedTo(Type t) {
        // foreign-domestic assignment is not implemented yet.
        if (t instanceof ForeignType) {
            throw new FeatureNotImplementedException(
                "Mixing your own classes with compiled classes.");
        } 
        if (! (t instanceof DomesticType) ) {
            return false;
        }
        // t must be a domestic type.
        // inheritance and interfaces are not implemented.
        return this == t;
    }
    

    /** Checks if a value of this type can be cast to a value of type
      * t in an explicit cast. 
      */
    public boolean mayBeCastTo(Type t) {
        // foreign-domestic assignment is not implemented yet.
        if (t instanceof ForeignType) {
            throw new FeatureNotImplementedException(
                "Mixing your own classes with compiled classes.");
        }   
        if (! (t instanceof DomesticType) ) {
            return false;
        }
        // t must be a domestic type.
        // inheritance and interfaces are not implemented.
        return this == t;
    }
    
    public MethodPointer getMethod(String name, Type[] argt) {
        PMethod meth = myClass.getMethod(name, argt);
        return meth.getPointer();
    }
    
    public FieldPointer getField(String name) {
        PVariableDeclarator field = myClass.getField(name);
        if (field == null) {
            throw new NonexistentFieldException(this, name);
        }
        return field.getPointer();
    }
    
    public String toString() {
        return myClass.getName();
    }
    
}
    
    