package jeliot.lang;

import jeliot.parser.*;
import jeliot.theatre.*;
import jeliot.FeatureNotImplementedException;

/**
  *
  * @author Pekka Uronen
  *
  * created         9.8.1999
  */
public class Value implements Cloneable {
    
    private Type type;

    /** An object describing the represented value. May be, for
      * example Integer, Char or Reference.
      */
    private Object val;
    private ValueActor actor;
    
    public Value(Type type, Object val) {
        this.type = type;
        this.val = val;
    }

    public static Value newBoolean(boolean b) {
        Boolean bool = b ? Boolean.TRUE : Boolean.FALSE;
        return new Value(PrimitiveType.booleanTYPE, bool);
    }
    
    public String toString() {
        return type.toString() + " " + val.toString();
    }

    public ValueActor getActor() {
        return actor;
    }

    public Object getValue() {
        return val;
    }

    public Type getType() {
        return type;
    }

    public void setActor(ValueActor actor) {
        this.actor = actor;
    }

    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Value cast(Type newtype) {
        Value casted = (Value)clone();
        casted.type = newtype;
        Type oldtype = this.type;
        if (oldtype == newtype) {
                casted.val = val;
        }
        else if (newtype.isNumeric()) {          
            Number numval = (val instanceof Character)  ?
                    new Integer(((Character)val).charValue())        :
                    (Number)val                         ;
                    
            if (newtype == PrimitiveType.intTYPE) {
                casted.val = new Integer(numval.intValue());
            }
            else if (newtype == PrimitiveType.doubleTYPE) {
                casted.val = new Double(numval.doubleValue());
            }
            else if (newtype == PrimitiveType.floatTYPE) {
                casted.val = new Float(numval.floatValue());
            }
            else if (newtype == PrimitiveType.longTYPE) {
                casted.val = new Long(numval.longValue());
            }
            else if (newtype == PrimitiveType.shortTYPE) {
                casted.val = new Short((short)numval.intValue());
            }
            else if (newtype == PrimitiveType.byteTYPE) {
                casted.val = new Byte((byte)numval.intValue());
            }
        }
        else if (newtype == PrimitiveType.charTYPE) {
            Number numval = (Number)val;
            casted.val = new Character((char)numval.intValue());
        }
        else {
            throw new FeatureNotImplementedException("Casting "+newtype);
        }
        
        return casted;
    }
                 
}
