package jeliot.parser;

import java.util.*;
import jeliot.parser.error.*;
import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.FeatureNotImplementedException;

/**
  *
  * @author Pekka Uronen
  *
  * created         30.9.1999
  */
public class PArrayCreation extends PExpression {
    
    private Vector expressions;
    private PType eType;
    private int dimension;
    
    private Type componentType; 
    
    /** Constructs a new array creation expression.
      * 
      *@param elementType The element type of the array.
      *@param exprs The dimension expressions of the array.
      *@param d     The dimension.
      */
    public PArrayCreation(PType elementType, Vector exprs, Integer d) {
        this.eType = elementType;
        this.expressions = exprs;
        this.dimension = (d == null) ? 0 : d.intValue();
        
        setParents(expressions);
    }
    
    /** Constructs a new array creation expression.
      * 
      */
    public PArrayCreation(PType elementType, Integer d, Vector init) {
        throw new FeatureNotImplementedException("Array initializer.");
    }
    
    public Type resolveType(NameSpace space, PClass context) {
        Type elementType = eType.resolveType(space, context);
        this.componentType = (dimension > 0) ?
                ArrayType.getType(elementType, dimension) :
                elementType;
        
        Type myType = ArrayType.getType(
                componentType, 
                expressions.size());
        setType(myType);
        return myType;
    }
    
    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterArrayCreation(this);
        if (expressions != null) {
            Enumeration enum = expressions.elements();
            while (enum.hasMoreElements()) {
                PExpression e = (PExpression)enum.nextElement();
                e.acceptVisitor(visitor);
            }
        }
        visitor.leaveArrayCreation(this);   
    }
    
    public Value evaluate(Director director) {
        super.evaluate(director);
        ArrayType atype = (ArrayType)getType();
        if (atype.getComponentType() instanceof ArrayType) {
            // multidimensional array...
            throw new FeatureNotImplementedException(
                    "Multidimensional arrays");
        }
        // only ONE dimension 
        PExpression expr = (PExpression)expressions.elementAt(0);
        Value dimensionValue = expr.evaluate(director);
        int dimension = ((Integer)dimensionValue.cast(
                PrimitiveType.intTYPE).getValue()).intValue();
        ArrayInstance array = new ArrayInstance(atype, dimension);
        director.showArrayCreation(array, dimensionValue);
        
        return new Reference(array);
    }
     
}
    
    