package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * PBlock represents a block of statements.
  *
  * @author Pekka Uronen
  *
  * created 5.8.1999
  */
public class PBlock extends PStatement {

    /** Statements included in this block */
    private Vector statements;

    /** Variables declared in this block */
    private Vector declVar;

    /** Number of variables declared in this block and this block's
      * subblocks It is set by compilationCheck method. */
    private int varCount;

    public PBlock(Vector statements, int left, int right) {
        super(left, right);
        this.statements = statements;

        setParents(statements);
    }
        
    public void acceptVisitor(ProgramTreeVisitor visitor) throws SemanticException {
        visitor.enterBlock(this);

        // recurse to all enclosed statements
        if (statements != null) {
            Enumeration enum = statements.elements();
            while (enum.hasMoreElements()) {
                PStatement s = (PStatement)enum.nextElement();
                s.acceptVisitor(visitor);
            }
        }
        visitor.leaveBlock(this);
    }
    
    void execute(Director director) throws Return {
        if (statements != null) {
            Enumeration enum = statements.elements();
            while (enum.hasMoreElements()) {
                PStatement s = (PStatement)enum.nextElement();
                s.execute(director);
            }
        }
    }

}
