package jeliot.parser;

import java.util.*;

/**
  * PCompilationUnit represents a compilation unit submitted to the
  * parser. It is the result of a succesful parse.
  *
  * @author Pekka Uronen
  *
  * created 5.8.1999
  */
public class PCompilationUnit extends PElement {

    /** The name of the package this compilation unit belongs to. */
    private PName myPackage;

    /** Imports declared in this compilation unit. */
    private Vector imports;
    
    /** Classes and interfaces declared in this compilation unit. */
    private Vector classes;

    /** This constructor is called by the parser. */
    PCompilationUnit(PName myPackage, Vector imports, Vector classes) {
        this.myPackage = myPackage;
        this.imports = imports;
        this.classes = classes;

        setParents(classes);
    }

    /** Lets given visitor object to traverse this compliation unit and
      * its contents.
      */
    public void acceptVisitor(ProgramTreeVisitor visitor) throws SemanticException {
        // tell the visitor to handle this compilation unit
        visitor.visitCompilationUnit(this);

        // recurse to all enclosed classes
        Enumeration enum = classes.elements();
        while (enum.hasMoreElements()) {
            PClass c = (PClass)enum.nextElement();
            c.acceptVisitor(visitor);
        }
    }

    /** Returns an enumeration of classes and interfaces declared in
      * this compilation unit. */
    public Enumeration getClasses() {
        return classes == null ?
                null :
                classes.elements();
    }

    /** Returns the name of the package this compilation units declares
      * to be in.
      */
    public String getPackageName() {
        return myPackage == null ?
                null :
                myPackage.toString();
    }

    /** Returns an enumeration of this compilation unit's import
      * declarations. */
    public Enumeration getImports() {
        return imports == null ?
                null :
                imports.elements();
    }
}
