package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * PLiteral represents a string, boolean, character, integral,
  * floating point or null literal.
  *
  * @author Pekka Uronen
  *
  * created         6.8.1999
  */
public class PLiteral extends PExpression {

    private Object value;
    private boolean negative;

    public PLiteral(Object value, int left, int right) {
        super(left, right);
        this.value = value;
        checkType(value);
    }

    private void checkType(Object o) {
        if (o == null) {
            setType(NullType.nullTYPE);
        }
        else if (o instanceof Boolean) {
            setType(PrimitiveType.booleanTYPE);
        }
        else if (o instanceof Integer) {
            setType(PrimitiveType.intTYPE);
        }
        else if (o instanceof Double) {
            setType(PrimitiveType.doubleTYPE);
        }
        else if (o instanceof String) {
            setType(ReferenceType.StringTYPE);
        }
        else if (o instanceof Character) {
            setType(PrimitiveType.charTYPE);
        }
        else if (o instanceof Long) {
            setType(PrimitiveType.longTYPE);
        }
        else if (o instanceof Float) {
            setType(PrimitiveType.floatTYPE);
        }
        else throw new RuntimeException(
            "Unknown literal type: " + o.getClass());
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.visitLiteral(this);
    }

    public Value evaluate(Director director) {
        super.evaluate(director);
        Value value = new Value(getType(), this.value);
        director.introduceLiteral(value);
        return value;
    }

    public String toString() {
        return getType() + " "+ value;
    }
    
    public boolean isPrimary() {
        return true;
    }

    public void negate() {
        this.negative = true;
        if (value instanceof Integer) {
            value = new Integer(-((Integer)value).intValue());
        }
        else if (value instanceof Double) {
            value = new Double(-((Double)value).doubleValue());
        }
        else if (value instanceof Float) {
            value = new Float(-((Float)value).floatValue());
        }
        else if (value instanceof Long) {
            value = new Long(-((Long)value).longValue());
        }
        
    }

    public boolean isProneToNegation() {
        return !negative && (value instanceof Number);
    }
}
