package jeliot.parser;

import java.util.*;

/**
  * PModifiers class encapsulates a simple or qualified Java name
  * (used in package declarations, field access, method invocations...)
  *
  * @author Pekka Uronen
  *
  * created 5.8.1999
  */
public class PName {

    public static final int AMBIGUOUS   = 0;
    public static final int TYPE        = 1;
    public static final int EXPRESSION  = 2;
    public static final int METHOD      = 3;
    public static final int PACKAGE     = 4;

    private String name;
    private int left;
    private int right;
    private PName prev;
    private PName tail;
    private int classification;

    public PName(String identifier, int left, int right) {
        this.name = identifier;
        this.left = left;
        this.right = right;
        this.tail = this;
    }

    public String getHead() {
        return name;
    }

    /** Adds an identifier to this name, making it a qualified name.
      * To be called by the parser. */
    public PName addIdentifier(String identifier, int left, int right) {
        PName head = new PName(identifier, left, right);
        head.prev = this;
        head.tail = tail;
        return head;
    }

    public boolean isQualified() {
        return prev != null;
    }

    public String toString() {
        int nl = name.length(); 
        int l = 0;
        PName p = prev;
        while (p != null) {
            l += p.name.length() +1;
            p = p.prev;
        }
        char[] ch = new char[l+nl];
        name.getChars(0, nl, ch, l); 
        p = prev;
        while (p != null) {
            ch [--l] = '.';
            l -= (nl = p.name.length()); 
            p.name.getChars(0, nl, ch, l);
            p = p.prev;
        }
        return new String(ch);
    }
    
    public int getLeftPos() {
        return tail.left;
    }

    public int getRightPos() {
        return right;
    }

    public int getClassification() {
        return classification;
    }

    public int classify(NameSpace space, PClass context) {
        if (prev == null) {
            // simple name

            // case 1: field access
            if (context.getField(name) != null) {
                return this.classification = TYPE;      
            }
            
            // case 2: type access
            
            if (space.findClass(name) != null) {
                return this.classification = TYPE;      
            }
            else {
                return this.classification = PACKAGE;
            }
        }
        else {
            int c = prev.classify(space, context);
            switch (c) {
                case (PACKAGE):
                    if (space.findClass(toString()) != null) {
                        return this.classification = TYPE;      
                    }
                    else {
                        return this.classification = PACKAGE;
                    }

                case (TYPE):
                case (EXPRESSION):
                    return this.classification = EXPRESSION;

            }
            throw new RuntimeException("Blaah.");
        }
            
    }
    

    /** Syntactic classification 6.5.1 */
    public void classify(int c) {
        this.classification = c;
        switch (c) {
            case (PACKAGE): 
                if (prev != null) {
                    prev.classify(PACKAGE);
                }
                break;
      
            case (TYPE):
                if (prev != null) {
                    prev.classify(PACKAGE);
                }
                break;

        }
    }

    public PName getLeftHand() {
        return prev;
    }

    public static void main(String[] args) {
        PName p = new PName("aapo", 1, 2);
        for (int i = 0; i < args.length; ++i) {
            p = p.addIdentifier(args[i], 1, 2);
        }
        System.out.println(p.toString());
    }

}
