package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * @author Pekka Uronen
  *
  * created         6.10.1999
  */
public class PReturn extends PStatement {

    private PExpression expr;
    private PMethod myMethod;
    
    /** Creates a return statement that returns no value.
      */
    public PReturn() { }
    
    /** Creates a return satement that returns the value of given 
      * expression.
      */
    public PReturn(PExpression expr, int left) {
        super(left, expr.getRightPos());
        this.expr = expr;
        expr.setParent(this);
    }
    
    public void setMethod(PMethod myMethod) {
        this.myMethod = myMethod;
    }
    
    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterReturn(this);
        if (expr != null) {
            expr.acceptVisitor(visitor);
        }
        visitor.leaveReturn(this);
    }
    
    void execute(Director director) throws Return {
        director.atStatement(this);
        if (expr != null) {
            Scratch scratch = director.openScratch();
            Value returnValue = expr.evaluate(director);
            Value casted = returnValue.cast(myMethod.getType());
            Return ret = new Return(casted);
            director.atStatement(this);
            director.animateReturn(ret, returnValue);
            director.closeScratch(scratch);
            throw ret;
        }
        throw new Return(this);
    }
}
