package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * PWhile represents a while statement.
  *
  * @author Pekka Uronen
  *
  * created         6.8.1999
  */
public class PWhile extends PStatement {

    private PExpression check;
    private PStatement statement;
    
    public PWhile(PExpression check, PStatement statement,
            int left, int right) {

		super(left, right);
		this.check = check;
		this.statement = statement;

		check.setParent(this);
		statement.setParent(this);
		
    }

    public PExpression getCheck() {
        return check;
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterWhile(this);
        check.acceptVisitor(visitor);
        statement.acceptVisitor(visitor);
        visitor.leaveWhile(this);
    }

    void execute(Director director) throws Return {
        // inform director we are at a new statement
        director.atStatement(this);
        Scratch scratch = director.openScratch();
        
        // evaluate the expression
        Value v = check.evaluate(director);
        boolean contLoop = ((Boolean)v.getValue()).booleanValue();

        // inform director about the result
        if ( contLoop ) {
            director.enterLoop(this, v);
        }
        else {
            director.skipLoop(this, v);
        }
        director.closeScratch(scratch);
        while ( contLoop ) {
            statement.execute(director);

            // back to this statement
            director.atStatement(this);
            scratch = director.openScratch();
            
            // evaluate the expression
            v = check.evaluate(director);
            contLoop = ((Boolean)v.getValue()).booleanValue();
            director.atStatement(this);
            if ( contLoop ) {
                director.continueLoop(this, v);
            }
            else {
                director.exitLoop(this, v);
            }
            director.closeScratch(scratch);
        }
    }

    public String statementName() {
        return "while";
    }

}
