package jeliot.parser.error;

import java.util.*;
import jeliot.lang.*;
import jeliot.parser.*;

/**
  * IllegalInvocationException is thrown when a type checker finds that
  * the compile time type of a variable access to invoke a method is a 
  * primitive type or void.
  * 
  * @author Pekka Uronen
  *
  * created         27.9.1999
  */
public class IllegalInvocationException extends SemanticException {
    
    PMethodInvocation invo;
    Type type;
    
    public IllegalInvocationException(Type t, PMethodInvocation invo) {
        this.type = t;
        this.invo = invo;
    }
    
    public String toString() {
         String typeStr = type.toString();
         String methName = invo.getIdentifier();
        
         return (
            "<HTML><HEAD><TITLE>XXX</TITLE></HEAD>\n" +
            "<BODY bgcolor=white>" +
            "<H1>Illegal method invocation.</H1>\n" +
            "<P>" +
            "Your program tries to invoke method <B>" +
            methName + "</B> in type <B>" + typeStr + "</B>." +
            "There is, however, no such method in type <B>" +
            typeStr + "</B>." + 
            "In fact, there are <B>no methods at all</B> in that " +
            "type. So please try to think a bit before pushing that " +
            "compile button next time, ok?" +
            "</BODY></HTML>"
            );
    }
    
}